@@ -1,3 +1,5 @@
+use std::io::{stdout, IsTerminal};
+
 /// Delta doesn't have a formal concept of a "theme". What it has is
 /// (a) the choice of syntax-highlighting theme
 /// (b) the choice of light-background-mode vs dark-background-mode, which determine certain
@@ -6,39 +8,38 @@
 /// by the user, it is determined by the classification of the syntax theme into light-background
 /// vs dark-background syntax themes. If the user didn't choose a syntax theme, a dark-background
 /// default is selected.
-use syntect::highlighting::ThemeSet;
+use bat;
+use bat::assets::HighlightingAssets;
 
-use crate::cli;
-use crate::env;
-use crate::utils::bat::assets::HighlightingAssets;
+use crate::cli::{self, DetectDarkLight};
 
 #[allow(non_snake_case)]
 pub fn set__is_light_mode__syntax_theme__syntax_set(
     opt: &mut cli::Opt,
     assets: HighlightingAssets,
 ) {
-    let syntax_theme_name_from_bat_theme = env::get_env_var("BAT_THEME");
+    let syntax_theme_name_from_bat_theme = &opt.env.bat_theme;
     let (is_light_mode, syntax_theme_name) = get_is_light_mode_and_syntax_theme_name(
         opt.syntax_theme.as_ref(),
         syntax_theme_name_from_bat_theme.as_ref(),
-        opt.light,
-        &assets.theme_set,
+        get_is_light(opt),
     );
     opt.computed.is_light_mode = is_light_mode;
 
     opt.computed.syntax_theme = if is_no_syntax_highlighting_syntax_theme_name(&syntax_theme_name) {
         None
     } else {
-        Some(assets.theme_set.themes[&syntax_theme_name].clone())
+        Some(assets.get_theme(&syntax_theme_name).clone())
     };
-    opt.computed.syntax_set = assets.syntax_set;
+    opt.computed.syntax_set = assets.get_syntax_set().unwrap().clone();
 }
 
 pub fn is_light_syntax_theme(theme: &str) -> bool {
     LIGHT_SYNTAX_THEMES.contains(&theme) || theme.to_lowercase().contains("light")
 }
 
-const LIGHT_SYNTAX_THEMES: [&str; 6] = [
+const LIGHT_SYNTAX_THEMES: [&str; 7] = [
+    "catppuccin-latte",
     "GitHub",
     "gruvbox-light",
     "gruvbox-white",
@@ -85,38 +86,59 @@ fn is_no_syntax_highlighting_syntax_theme_name(theme_name: &str) -> bool {
 fn get_is_light_mode_and_syntax_theme_name(
     theme_arg: Option<&String>,
     bat_theme_env_var: Option<&String>,
-    light_mode_arg: bool,
-    theme_set: &ThemeSet,
+    light_mode: bool,
 ) -> (bool, String) {
-    let theme_arg = valid_syntax_theme_name_or_none(theme_arg, theme_set);
-    let bat_theme_env_var = valid_syntax_theme_name_or_none(bat_theme_env_var, theme_set);
-    match (theme_arg, bat_theme_env_var, light_mode_arg) {
+    match (theme_arg, bat_theme_env_var, light_mode) {
         (None, None, false) => (false, DEFAULT_DARK_SYNTAX_THEME.to_string()),
-        (Some(theme_name), _, false) => (is_light_syntax_theme(&theme_name), theme_name),
-        (None, Some(theme_name), false) => (is_light_syntax_theme(&theme_name), theme_name),
+        (Some(theme_name), _, false) => (is_light_syntax_theme(theme_name), theme_name.to_string()),
+        (None, Some(theme_name), false) => {
+            (is_light_syntax_theme(theme_name), theme_name.to_string())
+        }
         (None, None, true) => (true, DEFAULT_LIGHT_SYNTAX_THEME.to_string()),
-        (Some(theme_name), _, is_light_mode) => (is_light_mode, theme_name),
-        (None, Some(theme_name), is_light_mode) => (is_light_mode, theme_name),
+        (Some(theme_name), _, is_light_mode) => (is_light_mode, theme_name.to_string()),
+        (None, Some(theme_name), is_light_mode) => (is_light_mode, theme_name.to_string()),
     }
 }
 
-// At this stage the theme name is considered valid if it is either a real theme name or the special
-// no-syntax-highlighting name.
-fn valid_syntax_theme_name_or_none(
-    theme_name: Option<&String>,
-    theme_set: &ThemeSet,
-) -> Option<String> {
-    match theme_name {
-        Some(name)
-            if is_no_syntax_highlighting_syntax_theme_name(name)
-                || theme_set.themes.contains_key(name) =>
-        {
-            Some(name.to_string())
-        }
-        _ => None,
+fn get_is_light(opt: &cli::Opt) -> bool {
+    get_is_light_opt(opt)
+        .or_else(|| should_detect_dark_light(opt).then(detect_light_mode))
+        .unwrap_or_default()
+}
+
+fn get_is_light_opt(opt: &cli::Opt) -> Option<bool> {
+    if opt.light {
+        Some(true)
+    } else if opt.dark {
+        Some(false)
+    } else {
+        None
     }
 }
 
+/// See [`cli::Opt::detect_dark_light`] for a detailed explanation.
+fn should_detect_dark_light(opt: &cli::Opt) -> bool {
+    match opt.detect_dark_light {
+        DetectDarkLight::Auto => opt.color_only || stdout().is_terminal(),
+        DetectDarkLight::Always => true,
+        DetectDarkLight::Never => false,
+    }
+}
+
+#[cfg(not(test))]
+fn detect_light_mode() -> bool {
+    use terminal_colorsaurus::{color_scheme, ColorScheme, QueryOptions};
+    color_scheme(QueryOptions::default()).unwrap_or_default() == ColorScheme::Light
+}
+
+#[cfg(test)]
+fn detect_light_mode() -> bool {
+    LIGHT_MODE_IN_TESTS
+}
+
+#[cfg(test)]
+pub(crate) const LIGHT_MODE_IN_TESTS: bool = false;
+
 #[cfg(test)]
 mod tests {
     use super::*;
